import pickle

import requests
from django.db import models
from google.auth.transport.urllib3 import Request
from requests_oauthlib import OAuth2
from urllib3 import PoolManager

import logging
from datetime import timedelta

import humanize

from davinci.caldav.models import CalDAVCalendar
from davinci.icalendar import ical_utils


class GoogleAuthInfo(models.Model):
    email = models.CharField(max_length=191, unique=True)
    _creds = models.BinaryField()

    def set_data(self, data):
        self._creds = pickle.dumps(data)

    def get_data(self):
        credentials = pickle.loads(self._creds)
        if credentials and credentials.expired and credentials.refresh_token:
            credentials.refresh(Request(http=PoolManager()))
            self._creds = pickle.dumps(credentials)
            self.save()
        return pickle.loads(self._creds)

    creds = property(get_data, set_data)

    def __str__(self):
        return self.email

    class Meta:
        verbose_name = "Google Authentication Information"
        verbose_name_plural = "Google Authentication Information"


class GCalSync(models.Model):
    name = models.CharField(max_length=191, verbose_name="Sync Name")
    calendar_id = models.CharField(max_length=191, verbose_name="Google Calendar ID",
                                   help_text="Can be found in the Google Calendar webapp, under "
                                             "'Calendar Settings' > 'Integrate calendar' > 'Calendar ID'")
    google_account = models.ForeignKey(to=GoogleAuthInfo, on_delete=models.CASCADE, verbose_name="Google Account")
    target = models.ForeignKey(to=CalDAVCalendar, on_delete=models.CASCADE, verbose_name="Target calendar")
    purge = models.BooleanField(verbose_name="Purge calendar (remove old events)", default=False,
                                help_text="Do not use if you are importing multiple ICS files into the same calendar.")
    sync_interval = models.DurationField(verbose_name="Sync interval", default=timedelta(hours=6))
    last_sync = models.DateTimeField(verbose_name="Last synchronised", blank=True, null=True)
    active = models.BooleanField(verbose_name="Sync active", default=True)

    class Meta:
        verbose_name = "Google Calendar Sync"
        verbose_name_plural = "Google Calendar Syncs"

    def __str__(self):
        return f'{self.name} to {self.target} every {humanize.precisedelta(self.sync_interval)} via {self.google_account}'

    @property
    def humanized_sync_interval(self):
        return humanize.precisedelta(self.sync_interval)

    def get_ical_data(self):
        logger = logging.getLogger("davinci.gcalendar.GCalSync.get_ical_data")
        creds = self.google_account.creds
        auth = OAuth2(client_id=creds.client_id, token={'access_token': creds.token, 'token_type': 'Bearer'})
        resp = requests.get(f"https://apidata.googleusercontent.com/caldav/v2/{self.calendar_id}/events", auth=auth)
        if resp.status_code == 200:
            return resp.content
        else:
            logger.error(f"Error while requesting events from Google for {self.name}: <HTTP {resp.status_code}> {resp.content}")


    def get_events(self):
        logger = logging.getLogger("davinci.gcalendar.GCalSync.get_events")
        data = self.get_ical_data()
        try:
            events = ical_utils.split_events(data)
        except ValueError as e:
            logger.error(f"ValueError while parsing events from iCal {self.name}: {e}.")
            return []

        # We have to pad the uids with some string related to the target calendar,
        # So that if someone wants to sync the same .ics to two calendars, the UIDs are different.
        fixed_events = ical_utils.fix_ical_uids(events, self.target)
        return fixed_events
